import logging
import os

log_dir = os.path.join(os.path.dirname(__file__), "..", "logs")
os.makedirs(log_dir, exist_ok=True)

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Create file handler
file_handler = logging.FileHandler(os.path.join(log_dir, "bsn.log"))
file_handler.setLevel(logging.DEBUG)

# Create console handler
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)

# Create formatter and add it to the handlers
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
file_handler.setFormatter(formatter)
console_handler.setFormatter(formatter)

# Add handlers to the logger
logger.addHandler(file_handler)
logger.addHandler(console_handler)



import os.path
import time

import pandas as pd
from pandas import DataFrame

from models import database
from models.models import YouTubeChannel
from src import logger
from youtube.youtube import (
    import_subscriptions,
    check_for_new_videos,
    calculate_interval_between_cycles,
)


def read_csv_file(
    file_path: str,
) -> DataFrame:
    """
    Reads a CSV file and returns a DataFrame or raises an error if the file is not found or is empty.
    :param file_path: str
        The path to the CSV file to be read.
    :return: DataFrame
        A pandas DataFrame containing the data from the CSV file.
    """
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"File {file_path} not found")
    df = pd.read_csv(file_path)

    if df.empty:
        raise ValueError(f"File {file_path} is empty")

    return df


def create_tables():
    with database:
        logger.info("Creating tables...")
        database.create_tables([YouTubeChannel])


def update_tables(yt_df: pd.DataFrame):
    logger.info(
        "Ensuring YouTube Channels table is up to date with subscriptions file..."
    )
    with database:
        import_subscriptions(yt_df)


def initialize(
    yt_subscriptions_file: str = "./data/subscriptions.csv",
    sqlite_db: str = "./data/bsn.db",
):
    yt_df = read_csv_file(yt_subscriptions_file)
    database.database = sqlite_db
    if not database.table_exists("youtubechannel"):
        logger.info("YouTube Channels table does not exist. Creating tables...")
        create_tables()

    update_tables(yt_df)


def main():
    logger.info("Staring BSN...")
    initialize()
    interval_between_checks: int = calculate_interval_between_cycles()

    while True:
        check_for_new_videos()
        logger.info(f"Sleeping for {interval_between_checks} seconds...")
        time.sleep(interval_between_checks)


if __name__ == "__main__":
    main()



import math
import os
from datetime import datetime, timedelta

import pandas as pd

from googleapiclient.errors import HttpError

from auth.youtube import get_youtube_service
from models.models import YouTubeChannel
from notifications.notifications import send_youtube_channels_notifications

from src import logger


def import_subscriptions(yt_df: pd.DataFrame):
    channel_ids = yt_df["Channel Id"].tolist()
    channels = get_channels_by_id(channel_ids)

    for channel in channels:
        channel_id = channel["id"]
        if not YouTubeChannel.select().where(YouTubeChannel.id == channel_id).exists():
            logger.info(
                f"Importing channel {channel_id} with {channel['statistics']['videoCount']} videos"
            )
            YouTubeChannel.create(
                id=channel["id"],
                num_videos=int(channel["statistics"]["videoCount"]),
            )


def get_channels_by_id(channel_ids: list[str]) -> list[dict] | None:
    channels: list[dict] = []

    for channel_str in _chunk_list(channel_ids):
        youtube = get_youtube_service()

        request = youtube.channels().list(part="statistics,snippet", id=channel_str)

        try:
            logger.info(f"Making request {request.uri}")
            response = request.execute()
            if "items" not in response:
                logger.warning(f"No items found with channel_ids: {channel_str}")
                return None
            channels.extend(response["items"])
        except HttpError as e:
            logger.error(
                f"An HTTP error {e.resp.status} occurred: {e.content.decode()} with channel_ids: {channel_str}"
            )
            return None

    return channels


def get_channels_with_new_videos(
    previous_channels: list[YouTubeChannel], current_channels: list[dict]
) -> list[dict]:
    new_video_channels = []

    for channel in current_channels:
        previous_channel = next(
            (c for c in previous_channels if c.id == channel["id"]), None
        )
        if int(channel["statistics"]["videoCount"]) > previous_channel.num_videos:
            logger.info(f"Channel {channel['id']} has new videos")
            new_video_channels.append(channel)
        elif int(channel["statistics"]["videoCount"]) < previous_channel.num_videos:
            logger.info(f"Video removed for channel {channel['id']}, updating channel")
            YouTubeChannel.update(
                num_videos=int(channel["statistics"]["videoCount"])
            ).where(YouTubeChannel.id == channel["id"]).execute()

    return new_video_channels


def update_channels(channels: list[dict]):
    for channel in channels:
        logger.info(
            f"Updating channel {channel['id']} with {channel['statistics']['videoCount']} videos"
        )
        YouTubeChannel.update(
            num_videos=int(channel["statistics"]["videoCount"])
        ).where(YouTubeChannel.id == channel["id"]).execute()


def check_for_new_videos():
    channels = YouTubeChannel.select()
    current_channels = get_channels_by_id([channel.id for channel in channels])
    new_video_channels = get_channels_with_new_videos(channels, current_channels)
    update_channels(new_video_channels)

    video = None
    if len(new_video_channels) > 0:
        if len(new_video_channels) == 1:
            video = get_most_recent_video(new_video_channels[0]["id"])

        send_youtube_channels_notifications(new_video_channels, video)


def get_most_recent_video(channel_id: str) -> dict | None:
    youtube = get_youtube_service()

    playlist_id = f"UU{channel_id[2:]}"

    request = youtube.playlistItems().list(
        part="snippet,status", maxResults=1, playlistId=playlist_id
    )

    try:
        logger.info(f"Making request {request.uri}")
        response = request.execute()
        if "items" not in response:
            logger.warning(f"No items found with channel_id: {channel_id}")
            return None

        video = response["items"][0]
        published_at = datetime.strptime(
            video["snippet"]["publishedAt"], "%Y-%m-%dT%H:%M:%SZ"
        )

        if (
            video["status"]["privacyStatus"] == "public"
            and video["snippet"]["position"] == 0
            and published_at >= (datetime.now() - timedelta(minutes=2))
        ):
            return video

    except HttpError as e:
        logger.error(
            f"An HTTP error {e.resp.status} occurred: {e.content} with channel_id: {channel_id}"
        )
        return None


def calculate_interval_between_cycles():
    num_channels: int = len(YouTubeChannel.select())
    num_api_keys: int = len(os.environ["YOUTUBE_API_KEYS"].split(","))
    max_requests_per_key_per_day = 10000
    total_requests_allowed_per_day = num_api_keys * max_requests_per_key_per_day
    requests_per_cycle = math.ceil((num_channels + 1) / 50)

    # Calculate the number of cycles we can perform in a day
    num_cycles_per_day = total_requests_allowed_per_day // requests_per_cycle

    # Total seconds in a day
    seconds_per_day = 24 * 60 * 60

    # Calculate the interval between each cycle
    interval_between_cycles = seconds_per_day / num_cycles_per_day

    return math.ceil(interval_between_cycles)


def _chunk_list(lst: list[str], chunk_size: int = 50) -> str:
    for i in range(0, len(lst), chunk_size):
        yield ",".join(lst[i : i + chunk_size])



from dotenv import load_dotenv
import os
from googleapiclient.discovery import build

from src import logger

load_dotenv()

# Set up the API key and service
API_KEYS = os.environ["YOUTUBE_API_KEYS"].split(",")
YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"
api_key_index = 0


def get_youtube_service():
    global api_key_index
    api_key = API_KEYS[api_key_index]
    api_key_index = (api_key_index + 1) % len(API_KEYS)
    logger.info(f"Connecting to YouTube API with key index {api_key_index}")
    return build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION, developerKey=api_key)



# create a peewee database instance -- our models will use this database to
# persist information
from peewee import Model, CharField, IntegerField
from models import database


class BaseModel(Model):
    class Meta:
        database = database


class YouTubeChannel(BaseModel):
    id = CharField(primary_key=True, unique=True, null=False)
    num_videos = IntegerField(null=False)



from apprise import apprise
from src import logger

from notifications import apprise_urls


def send_youtube_channels_notifications(channels: list[dict], video: dict = None):
    apobj = apprise.Apprise()

    for apprise_url in apprise_urls:
        apobj.add(apprise_url)

    if len(channels) > 1:
        title = f"{','.join([channel['snippet']['title'] for channel in channels])} have uploaded new videos to YouTube!"
        body = "Check them out here: https://www.youtube.com/feed/subscriptions"
    elif video:
        video_title = video["snippet"]["title"]
        video_url = f"https://www.youtube.com/watch?v={video['snippet']['resourceId']['videoId']}"
        title = (
            f"{channels[0]['snippet']['title']} has uploaded a new video to YouTube!"
        )
        body = f"{video_title}\n{video_url}"
    else:
        logger.warning("No video found")
        return

    logger.info(f"Sending notifications for {channels} channels")
    apobj.notify(title=title, body=body)
